# frozen_string_literal: true

require_relative 'xml/builder'
require_relative 'xml/default_list'
require_relative 'xml/default_map'
require_relative 'xml/doc_builder'
require_relative 'xml/error_handler'
require_relative 'xml/parser'
require_relative 'xml/parser/stack'
require_relative 'xml/parser/frame'
require_relative 'xml/parser/parsing_error'
