# frozen_string_literal: true

# WARNING ABOUT GENERATED CODE
#
# This file is generated. See the contributing guide for more information:
# https://github.com/aws/aws-sdk-ruby/blob/version-3/CONTRIBUTING.md
#
# WARNING ABOUT GENERATED CODE


module Aws::SSOOIDC
  # @api private
  module Endpoints


    def self.parameters_for_operation(context)
      Aws::SSOOIDC::EndpointParameters.create(context.config)
    end
  end
end
