# frozen_string_literal: true

module Aws
  autoload :DefaultsModeConfiguration, 'aws-defaults/default_configuration'
  autoload :DefaultsModeConfigResolver, 'aws-defaults/defaults_mode_config_resolver'
end
